import numpy as np
import cv2
from skimage.metrics import mean_squared_error

def calc_psnr(mse):
        if mse == 0:
                return 100
        PIXEL_MAX = 255.0
        return 10 * np.log10(PIXEL_MAX * PIXEL_MAX / mse)


def evalation(gt_light_field, reconstructed_img, args):
        view_point_num = args.viewpoint**2
        mse = np.zeros((view_point_num), np.float32)
        psnr = np.zeros((view_point_num), np.float32)

        for t in range(view_point_num):
                sub_mse = mean_squared_error(reconstructed_img[t], gt_light_field[t])
                sub_psnr = calc_psnr(sub_mse)
                mse[t] = sub_mse
                psnr[t] = sub_psnr

        return mse, psnr
