import numpy as np
import os
import cv2

def image_check(img_data_path, args):
        img = cv2.imread(img_data_path, cv2.IMREAD_UNCHANGED)
        if img is None:
                print("Error : file cannot open")
                print("File path is :: %s"%img_data_path)
                exit(1)

        if img.ndim == 2:
                _H, _W = img.shape
                chanel_num = 1
        else:
                _H, _W, chanel_num = img.shape

        H = _H - _H%(args.pix_mask_size)
        W = _W - _W%(args.pix_mask_size)

        return H, W, chanel_num

def test_lightfield_loader(frame, args, load_image_num):

        view_point_num = args.viewpoint**2

        fname = os.path.join(args.test_data, "%04d/%02d_%02d.png"%(frame, 0, 0))
        H, W, chanel_num = image_check(fname, args)

        lf_images = np.zeros([load_image_num, chanel_num, view_point_num, H, W], np.float32)

        for s in range(load_image_num):
                fname = os.path.join(args.test_data, "%04d/%02d_%02d.png"%(frame+s, 0, 0))
                H, W, chanel_num = image_check(fname, args)

                for t in range(view_point_num):
                        fname = os.path.join(args.test_data, "%04d/%02d_%02d.png"%(frame+s, t//args.viewpoint, t%args.viewpoint))
                        img = cv2.imread(fname, cv2.IMREAD_UNCHANGED)
                        if img is None:
                                print("Error : file cannot open")
                                print("File path is :: %s"%fname)
                                exit(1)

                        for c in range(chanel_num):
                                if chanel_num == 1:
                                        lf_images[s, c, t, :,:] = img[0:H, 0:W]
                                else:
                                        lf_images[s, c, t, :,:] = img[0:H, 0:W, c]

        return lf_images.astype(np.float32)
