import torch
import torch.nn.functional as F 
import torch.nn as nn 
	
class ApertureMask(nn.Module):
    def __init__(self):
        super(ApertureMask, self).__init__()
        self.cn01 = nn.Conv2d(25,1,1,padding=0)
        
    def forward(self, f0):
        f1 = self.cn01(f0)
        return f1

class PixelMask(nn.Module):
    def __init__(self):
        super(PixelMask, self).__init__()
        self.cn01 = nn.Conv2d(4,1,1,padding=0)
        
    def forward(self, f0):
        f1 = self.cn01(f0)
        return f1
    
class PixelSuffuleReconstruct(nn.Module):
    def __init__(self):
        super(PixelSuffuleReconstruct, self).__init__()
        self.conv1 = nn.Conv2d(64,64,1,padding=0)
        self.conv2 = nn.Conv2d(64,128,3,padding=1)
        self.conv3 = nn.Conv2d(128,128,3,padding=1)
        self.conv4 = nn.Conv2d(128,128,3,padding=1)
        self.conv5 = nn.Conv2d(128,256,3,padding=1)
        self.conv6 = nn.Conv2d(4,16,5,padding=2)
        self.conv7 = nn.Conv2d(16,64,5,padding=2)
        self.conv8 = nn.Conv2d(64,100,5,padding=2)

    def forward(self,f0):
        
        r = 8
        batchsize_f0 = f0.shape[0]
        in_ch_f0 = f0.shape[1]
        in_height_f0 = f0.shape[2]
        in_width_f0 = f0.shape[3]
        out_ch_f0 = int(in_ch_f0/(r**2))
        out_height_f0 = in_height_f0*r
        out_width_f0 = in_width_f0*r
        
        f1 = self.conv1(f0)
        f2 = self.conv2(f1)
        f3 = self.conv3(f2)
        f4 = self.conv4(f3)
        f5 = self.conv5(f4)

        batchsize_f5 = f5.shape[0]
        in_ch_f5 = f5.shape[1]
        in_height_f5 = f5.shape[2]
        in_width_f5 = f5.shape[3]
        out_ch_f5 = int(in_ch_f5/(r**2))
        out_height_f5 = in_height_f5*r
        out_width_f5 = in_width_f5*r

        f5_PS = torch.reshape(f5, (batchsize_f5,r,r,out_ch_f5, in_height_f5,in_width_f5))
        f5_PS = f5_PS.permute(0,3,4,1,5,2)
        f5_PS = torch.reshape(f5_PS, (batchsize_f5, out_ch_f5, out_height_f5, out_width_f5))

        f6 = self.conv6(f5_PS)
        f7 = self.conv7(f6)
        f8 = self.conv8(f7)

        return f8
        
    
class VeryDeepSuperResolution(nn.Module):
    def __init__(self):
        super(VeryDeepSuperResolution, self).__init__()
        self.cn01 = nn.Conv2d(100,100,3,padding=1)
        self.cn02 = nn.Conv2d(100,100,3,padding=1)
        self.cn03 = nn.Conv2d(100,100,3,padding=1)
        self.cn04 = nn.Conv2d(100,100,3,padding=1)
        self.cn05 = nn.Conv2d(100,100,3,padding=1)
        self.cn06 = nn.Conv2d(100,100,3,padding=1)
        self.cn07 = nn.Conv2d(100,100,3,padding=1)
        self.cn08 = nn.Conv2d(100,100,3,padding=1)
        self.cn09 = nn.Conv2d(100,100,3,padding=1)
        self.cn10 = nn.Conv2d(100,100,3,padding=1)
        self.cn11 = nn.Conv2d(100,100,3,padding=1)
        self.cn12 = nn.Conv2d(100,100,3,padding=1)
        self.cn13 = nn.Conv2d(100,100,3,padding=1)
        self.cn14 = nn.Conv2d(100,100,3,padding=1)
        self.cn15 = nn.Conv2d(100,100,3,padding=1)
        self.cn16 = nn.Conv2d(100,100,3,padding=1)
        self.cn17 = nn.Conv2d(100,100,3,padding=1)
        self.cn18 = nn.Conv2d(100,100,3,padding=1)
        self.cn19 = nn.Conv2d(100,100,3,padding=1)
        self.cnL = nn.Conv2d(100,100,3,padding=1)
        
    def forward(self, x):
        f = F.relu(self.cn01(x))
        f = F.relu(self.cn02(f))
        f = F.relu(self.cn03(f))
        f = F.relu(self.cn04(f))
        f = F.relu(self.cn05(f))
        f = F.relu(self.cn06(f))
        f = F.relu(self.cn07(f))
        f = F.relu(self.cn08(f))
        f = F.relu(self.cn09(f))
        f = F.relu(self.cn10(f))
        f = F.relu(self.cn11(f))
        f = F.relu(self.cn12(f))
        f = F.relu(self.cn13(f))
        f = F.relu(self.cn14(f))
        f = F.relu(self.cn15(f))
        f = F.relu(self.cn16(f))
        f = F.relu(self.cn17(f))
        f = F.relu(self.cn18(f))
        f = F.relu(self.cn19(f))
        f = self.cnL(f)
        return f + x
