import os
import math
import numpy as np
import torch
from PIL import Image


class SaveLog():

        def __init__(self, savedir, savename):
                self.savedir = savedir
                self.savename = savename
                self.savefile = os.path.join(savedir,savename)
                os.makedirs(savedir, exist_ok = True)

        def log(self, savetext):
                with open(self.savefile, mode='a') as f:
                        f.write(savetext)



class SaveMasks():

        def __init__(self, savedir):
                self.savedir = savedir
                os.makedirs(savedir, exist_ok = True)

        def save_mask_image(self, mask_data, savename, image_size):
                mask_image = Image.fromarray(np.uint8(mask_data*255.0))
                mask_image = mask_image.resize((image_size, image_size), Image.NEAREST)
                mask_image.save(savename)

        def save_aperture_mask(self, aperture_mask, mask_savedir, mask_savename, image_size=256):

                image_savedir = os.path.join(mask_savedir, "ApertureMask")
                os.makedirs(image_savedir, exist_ok=True)

                for n in range(len(aperture_mask)):
                        mask_data =  aperture_mask[n].cn01.weight.data[0,:,0,0].to("cpu")
                        mask_size = int(math.sqrt(len(mask_data)))
                        mask_data = mask_data.reshape(mask_size, mask_size)

                        fname = os.path.join(image_savedir, "%s%d.png"%(mask_savename, n+1))
                        self.save_mask_image(mask_data, fname, image_size=image_size)

        def save_pixel_mask(self, pixel_mask, shots_num, mask_savedir, mask_savename, image_size=256):

                image_savedir = os.path.join(mask_savedir, "PixelMask")
                os.makedirs(image_savedir, exist_ok=True)

                for s in range(shots_num):
                        mask_data = np.zeros(len(pixel_mask))
                        for n in range(len(pixel_mask)):
                                mask_data[n] = pixel_mask[n].cn01.weight.data[0,s,0,0].to("cpu")

                        mask_size = int(math.sqrt(len(mask_data)))
                        mask_data = mask_data.reshape(mask_size, mask_size)

                        fname = os.path.join(image_savedir, "%s%d.png"%(mask_savename, s+1))
                        self.save_mask_image(mask_data, fname, image_size=image_size)

        def save_masks(self, aperture_mask, pixel_mask):

                self.save_aperture_mask(aperture_mask, self.savedir, "ApertureMask")

                shots_num = len(aperture_mask)
                self.save_pixel_mask(pixel_mask, shots_num, self.savedir, "PixelMask")
