import numpy as np
import os
import torch
from model import *
import math


def apertureimg2subimg(img, shots_num, pix_mask_size, H, W):
        sub_H = int(H/pix_mask_size)
        sub_W = int(W/pix_mask_size)
        sub_time_images = torch.reshape(img,
                                    (img.shape[0],
                                     shots_num,
                                     sub_H,
                                     pix_mask_size,
                                     sub_W,
                                     pix_mask_size))

        sub_time_images = sub_time_images.permute(0,3,5,1,2,4)

        sub_time_images = torch.reshape(sub_time_images,
                                    (img.shape[0],
                                     pix_mask_size**2,
                                     shots_num,
                                     sub_H,
                                     sub_W))

        return sub_time_images


def subimg2acquiredimg(img, pix_mask_size, H, W):
        sub_H = int(H/pix_mask_size)
        sub_W = int(W/pix_mask_size)

        acquired_img = torch.reshape(img,
                                    (img.shape[0],
                                     pix_mask_size,
                                     pix_mask_size,
                                     sub_H,
                                     sub_W))

        acquired_img = acquired_img.permute(0,3,1,4,2)

        acquired_img = torch.reshape(acquired_img,
                                    (img.shape[0],
                                     H,
                                     W))

        return acquired_img


def image_shot(gray_lf_images, aperture_mask, pixel_mask, args):
        pix_mask_num = args.pix_mask_size**2
        view_point_num =  args.viewpoint**2
        H = gray_lf_images.shape[3]
        cut_H = H//args.shots_num
        W = gray_lf_images.shape[4]

        gray_sub_acquired_images = []
        gray_acquired_image = torch.zeros([H,W], dtype = torch.float32).to("cuda")

        for s in range(args.shots_num):
                gray_cut_light_field = gray_lf_images[:, :, :, cut_H*s : cut_H*(s+1), :]/255

                tmp_gray_aperture_image = []
                for ss in range(args.shots_num):
                        tmp_img = aperture_mask[(ss+s)%args.shots_num](gray_cut_light_field[ss+s])
                        tmp_gray_aperture_image.append(tmp_img)

                tmp_gray_aperture_image = tmp_gray_aperture_image[-s:]+tmp_gray_aperture_image[:-s]
                gray_aperture_image = torch.cat(tmp_gray_aperture_image, dim = 1)

                gray_sub_time_images = apertureimg2subimg(gray_aperture_image, args.shots_num, args.pix_mask_size, cut_H, W)

                tmp_sub_acquired_images = []
                for m in range(pix_mask_num):
                        noise = args.shots_num*view_point_num*args.dev*torch.randn(1,1,int(cut_H/args.pix_mask_size),int(W/args.pix_mask_size)).to("cuda")
                        tmp_sub_acquired_images.append(pixel_mask[m](gray_sub_time_images[:,m,:,:,:]) + noise)

                tmp_sub_acquired_images = torch.cat(tmp_sub_acquired_images, dim = 1)
                gray_sub_acquired_images.append(tmp_sub_acquired_images)

                tmp_gray_acquired_image = subimg2acquiredimg(tmp_sub_acquired_images, args.pix_mask_size, cut_H, W)
                gray_acquired_image[s*cut_H: (s+1)*cut_H, :] = tmp_gray_acquired_image.data[0,:,:]

        return gray_sub_acquired_images, gray_acquired_image



def image_reconstruct(gray_sub_acquired_image, PSreconstruct, vdsr, args):
        pix_mask_num = args.pix_mask_size**2
        view_point_num =  args.viewpoint**2
        H = gray_sub_acquired_image[0].shape[2]*args.pix_mask_size
        W = gray_sub_acquired_image[0].shape[3]*args.pix_mask_size

        gray_reconstruct_light_field = np.zeros((args.shots_num, args.shots_num, view_point_num, H, W), np.float32)

        for s in range(args.shots_num):
                tmp_reconstruct_light_field = vdsr[s](PSreconstruct[s](gray_sub_acquired_image[s]))
                tmp_reconstruct_light_field = torch.split(tmp_reconstruct_light_field, split_size_or_sections = int(tmp_reconstruct_light_field.shape[1]/args.shots_num), dim = 1)


                for ss in range(args.shots_num):
                        gray_reconstruct_light_field[s,ss,:,:,:] = tmp_reconstruct_light_field[ss].to("cpu").detach().numpy()[:,:,:]

        return gray_reconstruct_light_field


def image_concat(gray_cut_LF, gray_reconstruct_LF):
        shots_num = gray_cut_LF.shape[0]
        src_h = gray_cut_LF.shape[3]

        for s in range(shots_num):
                gray_reconstruct_LF[s:s+shots_num, : ,s*src_h:(s+1)*src_h, :]  = gray_cut_LF[s]

        return gray_reconstruct_LF
